<?php

/**
 * @file
 * ji_commerce_shipping.admin.inc
 *
 * Used to alter /admin/commerce/orders/%/edit screen.
 */

/**
 * Form callback for the add line item widget.
 *
 * Returns the elements necessary to add a shipping line item through the line
 * item manager widget.
 */
function ji_commerce_shipping_admin_line_item_add_form($form, &$form_state) {

  // Remove previous form fields.
  $form = array();

  // Collect the available shipping rates for this order.
  $order = $form_state['commerce_order'];

  // If possible load the order, since it sometimes is in a stale state.
  if (!empty($order->order_id)) {
    $order = commerce_order_load($order->order_id);
  }

  commerce_shipping_collect_rates($order);

  $form['shipping_rates'] = array(
    '#type' => 'value',
    '#value' => $order->shipping_rates,
  );

  // Create a list of all mandatory options.
  $mandatory_options = ji_commerce_shipping_service_rate_mandatory_options($order);
  if (sizeof($mandatory_options) > 0) {
    foreach ($mandatory_options as $key => $option) {
      $form['mandatory_shipping_rates'][$key] = array(
        '#type' => 'item',
        '#title' => t('Mandatory rates'),
        '#markup' => $option,
      );
    }

    foreach ($mandatory_options as $key => $option) {
      $form['mandatory_shipping_rates'][$key] += array(
        '#description' => $order->shipping_rates[$key]->data['shipping_service']['description'],
      );
    }
  }

  // Create a list of non-mandatory rates.
  $options = ji_commerce_shipping_service_rate_non_mandatory_options($order, $form_state);
  if (sizeof($options) > 0) {
    $form['shipping_service'] = array(
      '#type' => 'radios',
      '#title' => t('Shipping service'),
      '#options' => $options,
      '#default_value' => key($options),
    );
  }

  // No rates exist, inform user and enforce shipping rules.
  if (sizeof($mandatory_options) == 0 && sizeof($options) == 0) {
    $form['no_rates'] = array(
      '#type' => 'item',
      '#title' => t('Missing rates'),
      '#markup' => t('One or more products are missing rates.'),
    );
  }

  // Taxes.

  $options_default = array(0 => 'Select a county');

  foreach (commerce_tax_rates() as $key => $tax_rate) {
    $options[$key] = $tax_rate['display_title'] . ' - ' . ($tax_rate['rate'] * 100) . '%';
  }

  // Alphabetical order.
  asort($options);
  // 'Select a county' goes on top.
  $taxes = $options_default;
  $taxes += $options;

  $form['tax'] = array(
    '#type' => 'select',
    '#title' => t('Tax rate'),
    '#options' => $taxes,
    // We get $order object since that's where it's saved in the
    // validation callback.
    '#default_value' => '0',
    '#description' => t("Select your tax county."),
    '#required' => TRUE,
  );

  return $form;
}

/**
 * Validation function.
 *
 * Almost a direct copy of commerce_line_item_manager_validate() except
 * this now has a foreach loop that adds all applicable rates, mainly
 * those that are mandatory and the one the user selected.
 */
function ji_commerce_line_item_manager_validate($element, &$form_state, $form) {
  $value = array();

  // Loop through the line items in the manager table.
  foreach (element_children($element['line_items']) as $line_item_id) {
    // If the line item has been marked for deletion...
    if ($element['line_items'][$line_item_id]['remove']['#value']) {
      // Delete the line item now and don't include it from the $value array.
      commerce_line_item_delete($line_item_id);
    }
    else {
      // Add the line item ID to the current value of the reference field.
      $value[] = array('line_item_id' => $line_item_id);

      // Update the line item based on the values in the additional elements.
      $line_item = clone($element['line_items'][$line_item_id]['line_item']['#value']);

      // Validate the quantity of each line item.
      $element_name = implode('][', $element['line_items'][$line_item_id]['quantity']['#parents']);
      $quantity = $element['line_items'][$line_item_id]['quantity']['#value'];

      if (!is_numeric($quantity) || $quantity <= 0) {
        form_set_error($element_name, t('You must specify a positive number for the quantity'));
      }
      elseif ($element['line_items'][$line_item_id]['quantity']['#datatype'] == 'integer' &&
          (int) $quantity != $quantity) {
        form_set_error($element_name, t('You must specify a whole number for the quantity.'));
      }
      else {
        $line_item->quantity = $quantity;
      }

      // Manually validate the unit price of each line item.
      $unit_price = $form_state['values'][$element['#field_name']][$element['#language']]['line_items'][$line_item_id]['commerce_unit_price'];
      $amount = $unit_price[$element['#language']][0]['amount'];
      $currency_code = $unit_price[$element['#language']][0]['currency_code'];

      // Display an error message for a non-numeric unit price.
      if (!is_numeric($amount)) {
        $name = implode('][', array_merge($element['line_items'][$line_item_id]['commerce_unit_price']['#parents'], array($element['#language'], 0, 'amount')));
        form_set_error($name, 'You must enter a numeric value for the unit price.');
      }
      elseif ($amount <> $line_item->commerce_unit_price[$element['#language']][0]['amount'] ||
          $currency_code <> $line_item->commerce_unit_price[$element['#language']][0]['currency_code']) {
        // Otherwise update the unit price amount if it has changed.
        $line_item->commerce_unit_price = $unit_price;

        // Rebuild the price components array.
        commerce_line_item_rebase_unit_price($line_item);
      }

      // Only save if values were actually changed.
      if ($line_item != $element['line_items'][$line_item_id]['line_item']['#value']) {
        commerce_line_item_save($line_item);
      }
    }
  }

  // If the "Add line item" button was clicked, store the line item type in the
  // $form_state for the rebuild of the $form array.
  if (!empty($form_state['triggering_element'])) {
    if ($form_state['triggering_element']['#value'] === t('Add line item')) {
      $form_state['line_item_add'] = $element['actions']['line_item_type']['#value'];
      $form_state['rebuild'] = TRUE;
    }
    else {
      unset($form_state['line_item_add']);

      $parent = end($form_state['triggering_element']['#parents']);

      // If the save button was clicked from the line item type action form...
      if ($parent == 'save_line_item') {
        $line_item_type = $element['actions']['line_item_type']['#value'];

        // Extract an order ID from the form if present.
        $order_id = 0;

        if (!empty($form_state['commerce_order'])) {
          $order_id = $form_state['commerce_order']->order_id;
        }

        // Handles shipping and taxes.
        if ($line_item_type['type'] == 'shipping') {

          // In the case where only a mandatory item is available which means
          // no shipping service was selected by a user.
          $shipping_service = '';
          if (isset($element['actions']['shipping_service'])) {
            // Service that was selected by user, such as ups_3_day_select.
            $shipping_service = $element['actions']['shipping_service']['#value'];
          }

          if (isset($element['actions']['shipping_rates'])) {
            // Array list of all shippping rates.
            $shipping_rates = $element['actions']['shipping_rates']['#value'];

            $mandatory_shipping_rates = array();
            if (isset($element['actions']['mandatory_shipping_rates'])) {
              $mandatory_shipping_rates = $element['actions']['mandatory_shipping_rates'];
            }
            // Loop through each rate and add mandatory rate line items and
            // the one the user selected that wasn't mandatory such as
            // ups_3_day_select.
            foreach ($shipping_rates as $service_name => $rate) {
              if (array_key_exists($service_name, $mandatory_shipping_rates) || $service_name == $shipping_service) {
                $value[] = ji_commerce_shipping_add_shipping_rate($line_item_type, $order_id, $service_name, $rate, $form_state);
              }
            }

            // Loop through current line items and add tax to each one
            // per the user selected tax rate.
            foreach (element_children($element['line_items']) as $line_item_id) {
              $line_item = clone($element['line_items'][$line_item_id]['line_item']['#value']);

              // Clear away any previous tax amounts.
              $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
              ji_commerce_shipping_tax_remove_taxes($line_item_wrapper);

              // Add new tax amounts.
              $tax_name = $element['actions']['tax']['#value'];
              $tax_rate = commerce_tax_rate_load($tax_name);
              if ($tax_rate) {
                commerce_tax_rate_apply($tax_rate, $line_item);
              }

              commerce_line_item_save($line_item);
            }
          }
        }
        // We assume it's a $line_item_type['type'] == 'product'.
        else {
          // Create the new line item.
          $line_item = commerce_line_item_new($line_item_type['type'], $order_id);

          // If this type specifies a valid add form callback function...
          if ($callback = commerce_line_item_type_callback($line_item_type, 'add_form_submit')) {
            // Allow the callback to alter data onto the line item to be saved and
            // to return an error message if something goes wrong.
            // Calls commerce_product_line_item_add_form_submit.
            $error = $callback($line_item, $element, $form_state, $form);
          }
          else {
            // Assume no error if the callback isn't specified.
            $error = FALSE;
          }

          // If we didn't end up with any errors...
          if (empty($error)) {
            // Choose as default the tax rate we set for all products.
            // We override these if someone chooses to ship the products.
            if (!variable_get('ji_commerce_taxes_local_tax_rate')) {
              drupal_set_message(t('Set a local tax rate or else your orders will not have taxes applied.'), 'warning', FALSE);
            }
            else {
              $tax_name = variable_get('ji_commerce_taxes_local_tax_rate');
              $tax_rate = commerce_tax_rate_load($tax_name);
              if ($tax_rate) {
                commerce_tax_rate_apply($tax_rate, $line_item);
              }
            }

            // Save it and add it to the line item reference field's values array.
            commerce_line_item_save($line_item);

            // If the item is saved, we set a variable to notify the user the
            // need of saving the order.
            $form_state['line_item_save_warning'] = TRUE;

            $value[] = array('line_item_id' => $line_item->line_item_id);
          }
          else {
            // Otherwise display the error message; note this is not using
            // form_set_error() on purpose, because a failed addition of a line item
            // doesn't affect the rest of the form submission process.
            drupal_set_message($error, 'error');
          }
        }

        $form_state['rebuild'] = TRUE;
      }
      elseif ($parent == 'cancel') {
        // If the cancel button was clicked refresh without action.
        $form_state['rebuild'] = TRUE;
      }
    }
  }

  form_set_value($element, $value, $form_state);
}

/**
 * Helper function.
 *
 * Clear away all taxes located on each line item.
 */
function ji_commerce_shipping_tax_remove_taxes($line_item_wrapper) {
  // Load all the tax component types to look for matching price components in
  // the line item's unit price. Filter the list by tax rates that should be
  // removed if only some were specified.
  $component_types = commerce_tax_commerce_price_component_type_info();

  // Loop over the price components in the line item's unit price.
  $price = $line_item_wrapper->commerce_unit_price->value();

  foreach ($price['data']['components'] as $key => $component) {
    // Look for components that match one of the defined tax price components.
    if (in_array($component['name'], array_keys($component_types))) {
      // Remove it from the components array.
      unset($price['data']['components'][$key]);

      // If the component was marked as "included" in the price amount, update
      // the price amount to reflect the difference.
      if (!empty($component['included'])) {
        $price['amount'] -= $component['price']['amount'];
      }
    }
  }

  $line_item_wrapper->commerce_unit_price = $price;
}

/**
 * Helper function.
 *
 * Add a shipper rate to a commerce line item, returning it's id
 * so
 */
function ji_commerce_shipping_add_shipping_rate($line_item_type, $order_id, $service_name, $rate, &$form_state) {

  // Create the new line item.
  $line_item = commerce_line_item_new($line_item_type['type'], $order_id);
  $line_item->quantity = 1;
  // Use the label to store the display title of the shipping service.
  $line_item->line_item_label = commerce_shipping_service_get_title($service_name, 'display_title');
  $line_item->data['shipping_service'] = commerce_shipping_service_load($service_name);

  $rate_wrapper = entity_metadata_wrapper('commerce_line_item', $rate);
  $unit_price = $rate_wrapper->commerce_unit_price->value();

  // Set the service textfield and unit price.
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
  $line_item_wrapper->commerce_shipping_service = $service_name;
  $line_item_wrapper->commerce_unit_price = $unit_price;

  // Save it and add it to the line item reference field's values array.
  commerce_line_item_save($line_item);

  $form_state['line_item_save_warning'] = TRUE;

  return array('line_item_id' => $line_item->line_item_id);
}

function ji_commerce_shipping_add_tax_rate($line_item_type, $order_id, $service_name, &$form_state) {

  // Create the new line item.
  $line_item = commerce_line_item_new($line_item_type, $order_id);
  $line_item->quantity = 1;
  // Use the label to store the display title of the shipping service.
  $line_item->line_item_label = 'SALES TAX NAME'; //commerce_shipping_service_get_title($service_name, 'display_title');
  $line_item->data['sales_tax'] = commerce_tax_rate_load($service_name);

  $rate_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item->data['sales_tax']);
  $unit_price = $rate_wrapper->commerce_unit_price->value();

  // Set the service textfield and unit price.
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
  $line_item_wrapper->commerce_shipping_service = $service_name;
  $line_item_wrapper->commerce_unit_price = $unit_price;

  // Save it and add it to the line item reference field's values array.
  commerce_line_item_save($line_item);

  // If the item is saved, we set a variable to notify the user the
  // need of saving the order.
  // TODO-JI: Should re-enable auto saving.
  $form_state['line_item_save_warning'] = TRUE;

  return array('line_item_id' => $line_item->line_item_id);
}

/**
 * Callback: "submits" a new line item.
 *
 * Creates a new line item to the order.
 */
//function ji_commerce_shipping_admin_line_item_add_form_submit($line_item, $shipping_service, $shipping_rate, &$form_state, $form) {
  //dpm($form_state);
  //$order = $form_state['commerce_order'];
  //commerce_order_save($order);
  //drupal_set_message(t('Order saved succesfully.'));
//  // Ensure a quantity of 1.
//  $line_item->quantity = 1;
//
//  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $shipping_rate);
//  $unit_price = $line_item_wrapper->commerce_unit_price->value();
//
//  // Populate the line item with the appropriate data.
//  ji_commerce_shipping_admin_line_item_populate($line_item, $shipping_service, $unit_price);
//}

/**
 * Helper function.
 *
 * Adds information related to the shipping rate.
 */
//function ji_commerce_shipping_admin_line_item_populate($line_item, $service, $unit_price) {
//  // Use the label to store the display title of the shipping service.
//  $line_item->line_item_label = commerce_shipping_service_get_title($service, 'display_title');
//
//  // Store the full shipping status object in the data array.
//  $line_item->data['shipping_service'] = commerce_shipping_service_load($service);
//
//  // Set the service textfield and unit price.
//  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
//  $line_item_wrapper->commerce_shipping_service = $service;
//  $line_item_wrapper->commerce_unit_price = $unit_price;
//}

/**
 * Helper function.
 *
 * Returns the title of a shipping line item's related
 * shipping service.
 */
function ji_commerce_shipping_admin_line_item_title($line_item) {
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);

  // First try to get the title from the line item's data array.
  if (!empty($line_item->data['shipping_service']['display_title'])) {
    return $line_item->data['shipping_service']['display_title'];
  }
  // Then try to get the title from the shipping service.
  elseif ($line_item_wrapper->commerce_shipping_service->value() != NULL &&
      $title = commerce_shipping_service_get_title($line_item_wrapper->commerce_shipping_service->value(), 'display_title')) {
    return $title;
  }

  // Fallback to the line item label.
  return $line_item->line_item_label;
}
